/*
   The program "calls" adder.exe as an external process.
   This is a baby example of parallel programming, or a very
   simplified example of how a "remote procedure call" might work.
*/
#include <stdio.h>
#include <windows.h>

/* function prototype */
void printError(char* functionName);

int main(int argc, char *argv[])
{
   char commandLine[100];
   char env_var1[30];
   char env_var2[30];
   PROCESS_INFORMATION processInfo;

   STARTUPINFO startInfo;
   ZeroMemory(&startInfo, sizeof(startInfo));
   startInfo.cb = sizeof(startInfo);

   // get two command-line arguments
   int x1 = 0;
   int x2 = 0;
   if (argc >= 3)
   {
      x1 = atoi(argv[1]);
      x2 = atoi(argv[2]);
   }
   else
   {
      fprintf(stderr, "Usage: %s <number> <number>", argv[0]);
      exit(1);
   }

   /* pass arguments to adder by using environment variables */
   sprintf(env_var1, "adder-op1=%d", x1);
   sprintf(env_var2, "adder-op2=%d", x2);
   _putenv( env_var1 );
   _putenv( env_var2 );

   /* set up the command line */
   sprintf(commandLine, "%s", "adder.exe");

   /* create the child process */
   if( !CreateProcess(NULL, commandLine, NULL, NULL, TRUE,
                      NORMAL_PRIORITY_CLASS,
                      NULL, NULL, &startInfo, &processInfo) )
   {
       printError("CreateProcess");
   }

   /* wait for the "call" to adder to return */
   WaitForSingleObject(processInfo.hProcess, INFINITE);

   /* The result from adder.exe is already written to stdout */
   printf("\n");

   /* close all the handles */
   CloseHandle(processInfo.hThread);
   CloseHandle(processInfo.hProcess);

   _putenv("adder-op1=");
   _putenv("adder-op2=");

 return 0;
}

/****************************************************************
   The following function can be used to print out "meaningful"
   error messages. If you call a Win32 function and it returns
   with an error condition, then call this function right away and
   pass it a string containing the name of the Win32 function that
   failed. This function will print out a reasonable text message
   explaining the error and then (if chosen) terminate the program.
*/
void printError(char* functionName)
{
   LPVOID lpMsgBuf;
    int error_no;
    error_no = GetLastError();
    FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
    );
    // Display the string.
    fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
    fprintf(stderr, (const char*)lpMsgBuf);
    // Free the buffer.
    LocalFree( lpMsgBuf );
    //ExitProcess(1);  // terminate the program
}//printError
