/*
   The program "calls" adder as an external process.
   This is a baby example of parallel programming, or a very
   simplified example of how a "remote procedure call" might work.
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>

int main(int argc, char *argv[])
{
   // get two command-line arguments
   int x1 = 0;
   int x2 = 0;
   if (argc >= 3)
   {
      x1 = atoi(argv[1]);
      x2 = atoi(argv[2]);
   }
   else
   {
      fprintf(stderr, "Usage: %s <number> <number>\n", argv[0]);
      exit(1);
   }

   /* fork-exec the adder.c program */
   int fork_rv = fork();

   if ( -1 == fork_rv )  /* check for error */
   {
      perror("fork");
      exit(1);
   }
   else if ( 0 == fork_rv )  /* child */
   {
      char *env_var1 = malloc(30);
      char *env_var2 = malloc(30);

      /* pass arguments to adder by using environment variables */
      sprintf(env_var1, "adder-op1=%d", x1);
      sprintf(env_var2, "adder-op2=%d", x2);
      if ( putenv(env_var1) )
      {
         perror("putenv failed");
         exit(1);
      }
      if ( putenv(env_var2) )
      {
         perror("putenv failed");
         exit(1);
      }

      execl("./adder", "adder", NULL);
      perror("execl failed");
      exit(1);
   }
   else /* parent */
   {
      wait(NULL); /* wait for the child to complete */
      /* The result from adder is already written to stdout */
      printf("\n");
   }

 return 0;
}
