/*
   The program "calls" adder as an external process.
   This is a baby example of parallel programming, or a very
   simplified example of how a "remote procedure call" might work.
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>

int main(int argc, char *argv[])
{
   // get two command-line arguments
   int x1 = 0;
   int x2 = 0;
   if (argc >= 3)
   {
      x1 = atoi(argv[1]);
      x2 = atoi(argv[2]);
   }
   else
   {
      fprintf(stderr, "Usage: %s <number> <number>\n", argv[0]);
      exit(1);
   }

   /* fork-exec the adder.c program */
   int fork_rv = fork();

   if ( -1 == fork_rv )  /* check for error */
   {
      perror("fork");
      exit(1);
   }
   else if ( 0 == fork_rv )  /* child */
   {
      char arg1[30];
      char arg2[30];

      /* pass arguments to adder by using adder's command-line arguments */
      sprintf(arg1, "%d", x1);
      sprintf(arg2, "%d", x2);

      execl("./adder", "adder", arg1, arg2, NULL);
      perror("execl failed");
      exit(1);
   }
   else /* parent */
   {
      wait(NULL); /* wait for the child to complete */
      /* The result from adder is already written to stdout */
      printf("\n");
   }

 return 0;
}
