/*
   This program adds a number to every value read from
   standard input and writes the result to standard output.

   The default value to add to every input number is 0.
   This default value can be overridden by
      1.) a value from a configuration file, or by
      2.) an environment variable, or by
      3.) a command-line argument.
*/
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
   // Set the default value.
   int incBy = 0;


   // Override the default value with a value from the configuration file
   // (if it exists).
   FILE * fp;
   if ( (fp = fopen("incrementor.cfg", "r")) != NULL )
   {
      // get an operand from the configuration file
      fscanf(fp, "%d", &incBy);
   }


   // Override the default value with an environment variable value
   // (if it exists).
   char * op;
   if ( (op = getenv( "IncrementorValue" )) != NULL )
   {
      // get an operand from the environment
      incBy = atoi(op);
   }


   // Override the default value with a command-line argument
   // (if it exists).
   if (argc > 1)
   {  // get an operand from the command-line
      incBy = atoi(argv[1]);
   }

   // Process the stream of input numbers.
   double x;
   while( scanf("%lf", &x ) != EOF )
   {
      printf("%.f\n",  (x + incBy));
   }

   return 0;
}
